using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Styles;

using PortfolioFinance.DM;
using PortfolioFinance.BT;
using System.Data.OracleClient;
using System.Threading;
using PortfolioFinance.BO;
using System.Security.Principal;
using Dundas.Charting.WinControl;
using System.Diagnostics;
using System.Text.RegularExpressions;


namespace PortfolioFinanceWorkbench.UI
	{
	public partial class FormFeeChangeAnalysis : Product
		{
		// Toolstrip item controls
		#region attributes
		private ToolStripControlHost _dtStartcomponent;
		private ToolStripControlHost _dtEndcomponent;
		private ToolStripControlHost _btnTSrefresh;
		private Button _btngraph;

		private GridFilterBar _gridfilterbar;
		private DataSet _dsFees;
		private DataTable _dtNotionals;
		private Int32 _maxnotionalpct;

		private string _errormsg;
		private Int32 _currentcol;
		private Int32 _currentrow;

		// connection parameters
		private string _connstr;
		private Int32 _cmdtimeout;

		private Queue<FeeChangeRec> _feechangestack;
		private DateTime _minvaldate;
		private DateTime _minmoddate;

		#endregion

		public FormFeeChangeAnalysis()
			{
			InitializeComponent();

			_connstr = Properties.Settings.Default.ozconnstr;
			_cmdtimeout = Properties.Settings.Default.CommandTimeout;

			_dsFees = new DataSet();
			_dtNotionals = new DataTable();
			_currentcol = -1;
			_currentrow = -1;
			}

		public FormFeeChangeAnalysis( string text ) : this()
			{
            this.Text = text;
			}

		private void OnLoadFormFeeChangeAnalysis( object sender, EventArgs e )
			{
			this.Cursor = Cursors.WaitCursor;

			_gridFeeChanges.AllowSelection = GridSelectionFlags.Cell;

			_gridFeeChanges.GridVisualStyles = GridVisualStyles.Office2003;

		    _minvaldate = DateTime.Now;
			_minmoddate = DateTime.Now;

			_feechangestack =  new Queue<FeeChangeRec>();
			PortfolioFinance.BO.ClassEvnts.evExportData += new PortfolioFinance.BO.ClassEvnts.dgExportDataHandler( OnExportToExcel );

			AddToolstripItems();

			//create the filter bar and display it
			_gridfilterbar = new GridFilterBar();
			_gridfilterbar.CreatingColumnHeader += new GridFilterBarCreatingColumnHeaderEventHandler( OnGridFilterBarCreatingColumnHeader );
			_gridfilterbar.FilterBarFilterCompleted +=new GridFilterBarFilterCompletedEventHandler( OnGridFilterBarFilterCompleted );

			_tbNotional.Enabled = false;

			_dtpstartdate.Value = DateTime.Now;
			_dtpstartdate.MaxDate = _dtpstartdate.Value;
			_gridFeeChanges.ContextMenuStrip = _swapcnxtmenustrip;
			
			LoadGrid( _dtpstartdate.Value.ToString( "MM/dd/yyyy" ) );

			_dtpstartdate.MinDate = _minvaldate;
			_dtpstartdate.MaxDate = DateTime.Now;
			_dtpstartdate.Value = DateTime.Now.Subtract( new TimeSpan( 30,0,0,0) );

			_dtpenddate.MinDate = _minvaldate;

			_gridfilterbar.RowFilter = GetDateFilterSpan( _dtpstartdate.Value, _dtpenddate.Value );

			_dtpstartdate.Leave += new EventHandler( OnStartDateLeaveFocus );
			_dtpenddate.Leave += new EventHandler( OnEndDateLeaveFocus );
			_dtpstartdate.ValueChanged += new EventHandler( OnStartDateLeaveFocus );
			_dtpenddate.ValueChanged += new EventHandler( OnEndDateLeaveFocus );
			
			_gridFeeChanges.Paint +=new System.Windows.Forms.PaintEventHandler(OnGridFeeChangesPaint);
			
			this.Cursor = Cursors.Default;
			}

		private void AddToolstripItems()
			{
			_dtStartcomponent = new ToolStripControlHost( _dtpstartdate );
			_dtpstartdate.Cursor = Cursors.Hand;
			_tsswap.Items.Add( "Fee Start Date" );
			_tsswap.Items.Add( _dtStartcomponent );

			_dtpenddate.Value = DateTime.Now;
			_dtEndcomponent = new ToolStripControlHost( _dtpenddate );
			_dtpenddate.CustomFormat = "MM/dd/yyyy";
			_dtpenddate.Cursor = Cursors.Hand;
			_tsswap.Items.Add( "Fee End Date" );
			_tsswap.Items.Add( _dtEndcomponent );

			_btngraph = new Button();
			_btngraph.Cursor = Cursors.Hand;
			_btngraph.Visible = false;

			//_btngraph.BackgroundImage = ( Image ) Properties.Resources.graph;
			//_btngraph.Size = new Size( 32, 32 );
			//_btnTScomponent = new ToolStripControlHost( _btngraph );
			//_btnTScomponent.ToolTipText = "Show/Hide Graph";
			//_tsswap.Items.Add( _btnTScomponent );

			//_btngraph.Click += new EventHandler( OnGraphButtonClick );

			_panelgraph.Visible = false;
			}

		private void SetTableIntoGridAndWireFilterBar( DataTable dt )
			{
			
			this.Cursor = Cursors.WaitCursor;

			DataView dv = new DataView( dt );
			dv.Sort = "symbol,cpy,value_date";

			_gridFeeChanges.EnableEdit = false;
			_gridFeeChanges.ControllerOptions = GridControllerOptions.All;

			_gridFeeChanges.GridBoundColumns.Clear();
			_gridFeeChanges.Selections.Clear();
			_gridFeeChanges.BeginUpdate();

			_gridFeeChanges.DataSource = dv;

			_gridFeeChanges.Model.ColWidths.ResizeToFit( GridRangeInfo.Row( 0 ), GridResizeToFitOptions.NoShrinkSize );

			GridStyleInfo style = new GridStyleInfo();
			style.ModifyStyle( _gridFeeChanges.BaseStylesMap[ "Header" ].StyleInfo, StyleModifyType.Copy );

			// Set cell type of the style object to ComboBox.

			style.CellType = "ComboBox";
			style.ExclusiveChoiceList = true;
			// Apply style settings.
			style.BaseStyle = "Standard";
			style.Font.Bold = false;
			style.BackColor = _gridFeeChanges.TableStyle.BackColor;
			style.Borders.Bottom = new GridBorder( GridBorderStyle.Solid );
			// Set DropDownStyle to Autocomplete.
			style.DropDownStyle = GridDropDownStyle.AutoComplete;

			_gridFeeChanges.BaseStylesMap[ "Header" ].StyleInfo = style;

			//catch the creating column header event so we can turn off unwanted filter headers
			//needs to be hooked before WireGrid call to catch things...
			_gridfilterbar.WireGrid( this._gridFeeChanges );

			FormatGridForTable();

			_gridFeeChanges.EndUpdate();

			this.Cursor = Cursors.Arrow;
			}

		private void FormatGridForTable()
			{
			GridBoundColumnsCollection gbcc = _gridFeeChanges.Binder.InternalColumns;
			gbcc[ "symbol" ].HeaderText = "Symbol";
			gbcc[ "cpy" ].HeaderText = "Counterparty";
			gbcc[ "fee" ].HeaderText = "Fee";
			
			gbcc[ "value_date" ].HeaderText = "Fee Date";
			gbcc[ "value_date" ].StyleInfo.Format = "yyyy/MM/dd";
			gbcc[ "value_date" ].StyleInfo.CellValueType = typeof( DateTime );
			gbcc[ "value_date" ].StyleInfo.CellType = "TextBox";

			gbcc[ "last_update" ].HeaderText = "Last Update DateTime";
			gbcc[ "last_update" ].StyleInfo.Format = "yyyy/MM/dd HH:mm:ss";
			gbcc[ "last_update" ].StyleInfo.CellValueType = typeof( DateTime );
			gbcc[ "last_update" ].StyleInfo.CellType = "TextBox";
			}

		private void LoadGrid( string knowledgedate )
			{
			this.Cursor = Cursors.WaitCursor;
			
			DataSet ds  = RetrieveFeeData( knowledgedate );
			DataTable dt = FindFeeChanges( ds );
			
			_gridFeeChanges.Refresh();
			SetTableIntoGridAndWireFilterBar( dt );

			this.Cursor = Cursors.Default;
			}


		#region Event Handlers
		private void OnRefreshClick( object sender, EventArgs e )
			{
			//theFilterBar.RowFilter = "([VALUE_DATE] >= '2008/10/15' AND [VALUE_DATE] <= '2008/10/31')";

			// Loading Chart Data Section
			this.Cursor = Cursors.WaitCursor;


			this.Cursor = Cursors.Default;
			}

		private void OnGraphButtonClick( object sender, EventArgs e )
			{
			_panelgraph.Visible = !( _panelgraph.Visible );
			}

		private void OnStartDateLeaveFocus( object sender, EventArgs e )
			{
			_gridfilterbar.RowFilter = GetDateFilterSpan( _dtpstartdate.Value, _dtpenddate.Value );
			_gridFeeChanges.Refresh();
			}

		private void OnEndDateLeaveFocus( object sender, EventArgs e )
			{
			_gridfilterbar.RowFilter = GetDateFilterSpan( _dtpstartdate.Value, _dtpenddate.Value );
			_gridFeeChanges.Refresh();
			}

		private void OnGridFeeChangeAnalysisCellClick( object sender, GridCellClickEventArgs e )
			{
			_currentcol = e.ColIndex;
			_currentrow = e.RowIndex;
			if ( _currentrow == 0 && e.MouseEventArgs.Button == MouseButtons.Right )
				{
				_gridFeeChanges.Invalidate();
				_gridFeeChanges.ContextMenuStrip.Visible = true;
				}
			else
				{
				_gridFeeChanges.ContextMenuStrip.Visible = false;
				}
			}

		private void OnGridFeeChangeAnalysisControlMouseDown( object sender, CancelMouseEventArgs e )
			{
			if ( e.MouseEventArgs.Button == MouseButtons.Right )
				{
				e.Cancel = true;
				_gridFeeChanges.ContextMenuStrip.Visible = false;
				}
			}

		private void OnGridFilterBarCreatingColumnHeader( object sender, GridFilterBarCreatingColumnHeaderEventArgs e )
			{
			if ( e.ColName.ToLower() == "fee" ) e.Cancel = true;
			if ( e.ColName.ToLower() == "value_date" ) e.Cancel = true;
			if ( e.ColName.ToLower() == "last_update" ) e.Cancel = true;
			}

		private void OnGridFilterBarFilterCompleted( object sender, GridFilterBarTextChangedEventArgs e )
			{
			_gridfilterbar.FilterBarFilterCompleted -= OnGridFilterBarFilterCompleted ;
			_gridfilterbar.RowFilter = GetDateFilterSpan( _dtpstartdate.Value, _dtpenddate.Value );
			_gridFeeChanges.Refresh();
			_gridfilterbar.FilterBarFilterCompleted += new GridFilterBarFilterCompletedEventHandler( OnGridFilterBarFilterCompleted );
			}

		private void OnGridFeeChangeAnalysis_PrepareViewStyleInfo( object sender, GridPrepareViewStyleInfoEventArgs e )
			{
			GridDataBoundGrid grid = sender as GridDataBoundGrid;
			if ( grid != null && e.RowIndex > grid.Model.Rows.HeaderCount && e.ColIndex > grid.Model.Cols.HeaderCount )
				{
				if ( grid.IsShowCurrentRow( e.RowIndex ) && !grid.CurrentCell.HasCurrentCellAt( e.RowIndex, e.ColIndex ) )
					{
					e.Style.BackColor = SystemColors.Highlight;
					e.Style.TextColor = SystemColors.HighlightText;
					}
				else
					{
					GridBoundRecordState rs = grid.Binder.GetRecordStateAtRowIndex( e.RowIndex );
					if ( rs.Position % 2 != 0 )
						e.Style.BackColor = Color.FromArgb( 219, 226, 242 );
					}
				}
			}

		private void OnHideColumnsToolStripMenuItemClick( object sender, EventArgs e )
			{
			int index = _currentcol; ;
			if ( index != -1 ) _gridFeeChanges.SetColHidden( index, index, true );
			}

		private void OnUnhideColumnsToolStripMenuItemClick( object sender, EventArgs e )
			{
			GridBoundColumnsCollection collect = _gridFeeChanges.GridBoundColumns;
			_gridFeeChanges.SetColHidden( 0, _gridFeeChanges.GridCellsRange.Width, false );
			}

		private void OnTaskBarNotionalScroll( object sender, EventArgs e )
			{
			TrackBar tb = ( TrackBar ) sender;
			Int32 pos = tb.Value;
			Int32 pctval = 0;

			if ( pos == 0 )
				pctval = _maxnotionalpct;
			else if ( pos == 40 )
				pctval = 0;
			else pctval = _maxnotionalpct / pos;

			UpdateChartData( pctval );
			}

		private void OnGridFeeChangesPaint( object sender, PaintEventArgs e )
			{
			//Color rowColor = Color.Black;
			//Brush circleColor = new SolidBrush( rowColor );
			//String drawString = "\xFC";
			//SolidBrush drawBrush = new SolidBrush( Color.Black );
			//StringFormat drawFormat = new StringFormat();
			//drawFormat.FormatFlags = StringFormatFlags.DirectionVertical;
			//Font drawFont = new Font( "Wingdings", 10, FontStyle.Bold );

			//SizeF Sz = e.Graphics.MeasureString( drawString, drawFont );

			//e.Graphics.DrawString( drawString, drawFont, SystemBrushes.ControlText, e.ClipRectangle.Location.X + 15,
			//    e.ClipRectangle.Location.Y + ( ( e.ClipRectangle.Height - Sz.Height ) / 2 ) );
			}

		#endregion

		#region Graphing Section

		private void CreateChart()
			{
			string series1 = " Long Notional Pct";
			string series2 = " Short Notional Pct";

			_chartHigh.ChartAreas[ 0 ].AxisX.LabelStyle.FontAngle = -45;
			_chartHigh.ChartAreas[ 0 ].AxisX.Interval = 1;
			_chartHigh.Series[ 0 ].Name = series1;
			_chartHigh.Series[ 1 ].Name = series2;

			// no data for grid so binding is not possible >> return;
			if ( _dsFees.Tables.Count == 0 ) return;

			_dtNotionals = GetChartData();

			if ( _dtNotionals.Rows.Count > 0 )
				UpdateChartData( _maxnotionalpct );
			}

		private void UpdateChartData( Int32 pctnotional )
			{
			string selectFilter = string.Format( "(shortnotionalpct >={0:N} ) or ( longnotionalpct >= {1:N} )", pctnotional, pctnotional );

			DataView dvNotional = _dtNotionals.DefaultView;

			_tbNotional.Enabled = ( dvNotional.Count > 0 );

			// If no data in the view then return
			if ( dvNotional.Count == 0 ) return;

			dvNotional.RowFilter = selectFilter;

			// Set up scale breaks
			// Enable scale breaks
			_chartHigh.ChartAreas[ "Default" ].AxisY.ScaleBreakStyle.Enabled = true;

			// Set the scale break type
			_chartHigh.ChartAreas[ "Default" ].AxisY.ScaleBreakStyle.BreakLineType = BreakLineType.Wave;

			// Set the spacing gap between the lines of the scale break (as a percentage of y-axis)
			_chartHigh.ChartAreas[ "Default" ].AxisY.ScaleBreakStyle.Spacing = 3;

			// Set the line width of the scale break
			_chartHigh.ChartAreas[ "Default" ].AxisY.ScaleBreakStyle.LineWidth = 1;

			// Set the color of the scale break
			_chartHigh.ChartAreas[ "Default" ].AxisY.ScaleBreakStyle.LineColor = Color.Blue;

			// Show scale break if more than 25% of the chart is empty space
			_chartHigh.ChartAreas[ "Default" ].AxisY.ScaleBreakStyle.CollapsibleSpaceThreshold = 25;

			// If all data points are significantly far from zero, 
			// the Chart will calculate the scale minimum value
			_chartHigh.ChartAreas[ "Default" ].AxisY.ScaleBreakStyle.StartFromZero = AutoBool.Auto;

			// Bind Grid Data to chart
			_chartHigh.Series[ 0 ].Points.DataBindXY( dvNotional, "broker", dvNotional, "longnotionalpct" );
			_chartHigh.Series[ 1 ].Points.DataBindXY( dvNotional, "broker", dvNotional, "shortnotionalpct" );
			}
	

		#endregion


		private void OnExportToExcel( Control ctr )
			{
			Syncfusion.GridExcelConverter.GridExcelConverterControl gecc = new Syncfusion.GridExcelConverter.GridExcelConverterControl();

			SaveFileDialog saveFileDialog = new SaveFileDialog();
			saveFileDialog.Filter = "Files(*.xls)|*.xls";
			saveFileDialog.DefaultExt = ".xls";

			if ( saveFileDialog.ShowDialog() == DialogResult.OK )
				{
				gecc.GridToExcel( _gridFeeChanges.Model, saveFileDialog.FileName,
					Syncfusion.GridExcelConverter.ConverterOptions.RowHeaders );

				if ( MessageBox.Show( "Do you wish to open the xls file now?", "Export to Excel", MessageBoxButtons.YesNo, MessageBoxIcon.Question ) == DialogResult.Yes )
					{
					Process proc = new Process();
					proc.StartInfo.FileName = saveFileDialog.FileName;
					proc.Start();
					}
				}
			}

		#region GridFilter Helper Methods
		private string GetDateFilterSpan( DateTime startdate, DateTime enddate )
			{
			string start = startdate.ToString( "yyyy/MM/dd" );
			string end = enddate.ToString( "yyyy/MM/dd" );
			string newrowfilter = string.Format( "([VALUE_DATE] >= '{0}' AND [VALUE_DATE] <= '{1}')", start, end );

			// If the filter bar is empty then this is a new filter;
			if ( _gridfilterbar.RowFilter == null || _gridfilterbar.RowFilter == "" ) return newrowfilter;

			string rowfilter = ParseFilter( _gridfilterbar.RowFilter );

			if ( ( rowfilter != null ) &&
					( rowfilter.Contains( "CPY" ) || rowfilter.Contains( "SYMBOL" ) || rowfilter.Contains( "CCY" ) )
				)
				{
				newrowfilter = rowfilter + " and " + newrowfilter;
				}

			return newrowfilter;
			}

		// <summary>
		// <para>([SYMBOL] = '1109 HK') and ([CPY] = 'BARC') and ([VALUE_DATE] >= '{0}' AND [VALUE_DATE] <= '{1}')</para>
		// </summary>

		private string ParseFilter( string gridfilter )
			{
			string pattern = @"(?:(\(\[(?:SYMBOL|CPY|CCY)\] += +\'[A-Z,0-9, ]*\'\))(?: +and +)?)+";
			Match match = null;
			string tempstr = null;

			Regex regex = new Regex( pattern, RegexOptions.Compiled | RegexOptions.RightToLeft );
			match = regex.Match( gridfilter );
			
			if ( match.Success == false ) return null;

			foreach (  Capture capture in match.Groups[1].Captures )
				{
				if ( tempstr == null )  
					tempstr = capture.Value;
				else
					tempstr += " and " + capture.Value;
				}
			
			return tempstr;
			}

		#endregion

		#region Data Access Area

		private DataTable GetChartData()
			{

			DataSet ds = SwapNotionalPct( _dtpstartdate.Value );

			_dtNotionals = ds.Tables[ 0 ];

			_tbNotional.Enabled = ( _dtNotionals.Rows.Count > 0 );

			if ( _dtNotionals.Rows.Count == 0 ) return _dtNotionals;

			_maxnotionalpct = ( short ) ( double ) _dtNotionals.Compute( "Max(longnotionalpct)", "longnotionalpct>0" );

			if ( ( ( short ) ( double ) _dtNotionals.Compute( "Max(shortnotionalpct)", "shortnotionalpct>0" ) ) > _maxnotionalpct )
				_maxnotionalpct = ( short ) ( double ) _dtNotionals.Compute( "Max(shortnotionalpct)", "shortnotionalpct>0" );

			_tbNotional.Value = 0;
			return _dtNotionals;
			}

		private DateTime GetCurrentKnowledgeDate()
			{
			DataSet ds = new DataSet();
			string procname = Properties.Settings.Default.GetSysControlValue;
			string connstr = Properties.Settings.Default.fincontrolsconnstr;
			Int32 cmdtimeout = Properties.Settings.Default.CommandTimeout;
			DateTime resultval = DateTime.Now;
			try
				{
				PFSqlParameters myparams = new PFSqlParameters();
				myparams.AddInputParameter( "@name", "LastBusinessDate" );
				PFSqlExecute excmd = new PFSqlExecute();
				string val = ( string ) excmd.ExecuteScalar( procname, true, connstr, myparams.Parameters );
				string[] datefields = val.Split( '/' );

				int month = Convert.ToInt16( datefields[ 0 ] );
				int day = Convert.ToInt16( datefields[ 1 ] );
				int year = Convert.ToInt16( datefields[ 2 ] );				
				resultval = new DateTime( year, month, day );
				}
			catch ( SystemException ex )
				{
				_errormsg = ex.Message;
				
				MessageBox.Show( _errormsg, 
													"FormFeeChangeAnalysis:GetCurrentKnowledgeDate() Error", 
													MessageBoxButtons.OK, 
													MessageBoxIcon.Exclamation );
				}
			return resultval;
			}

		private DataSet RetrieveFeeData( string knowledgedate )
			{
			DataSet ds = new DataSet();
			string procname = Properties.Settings.Default.FeeChangeAnalysisSql;

			try
				{
				PFOraParameters myparams = new PFOraParameters();
				myparams.AddOutputParameterEx( "cur_fees" );
				myparams.ParameterMap[ "cur_fees" ].OracleType = System.Data.OracleClient.OracleType.Cursor;

				PFOraExecute excmd = new PFOraExecute();
				excmd.FillDataSet( procname, true, _connstr, ds, myparams.Parameters );
				}
			catch ( SystemException ex )
				{
				_errormsg = ex.Message;
				MessageBox.Show("FormFeeChangeAnalysis:RetrieveFeeData() Error Reading from Oracle Database", "Data Error",MessageBoxButtons.OK, MessageBoxIcon.Error );
				}
			
			return ds;
			}

		private DataTable FindFeeChanges( DataSet ds )
			{
			foreach ( DataRow row in ds.Tables[ 0 ].Rows )
				{
				string symbol = Convert.ToString( row[ "symbol" ] );
				string cpy = Convert.ToString( row[ "cpy" ] );
				string ccy = Convert.ToString( row[ "ccy" ] );
				double fee = Convert.ToDouble( row[ "fee" ] );
				DateTime valuedate = Convert.ToDateTime( row[ "value_date" ] );
				DateTime lastupdate = Convert.ToDateTime( row[ "last_update" ] );
				FeeChangeRec fc = new FeeChangeRec( symbol, cpy, ccy, fee, valuedate, lastupdate );

				if ( _feechangestack.Count == 0 )
					{
					_feechangestack.Enqueue( fc );
					continue;
					}

				FeeChangeRec prevfee = ( FeeChangeRec ) _feechangestack.ToArray()[ _feechangestack.Count-1 ];
				if ( ComparePreviousFee( prevfee, fc ) == false ) _feechangestack.Enqueue( fc );
				}

			ds.Tables[ 0 ].Clear();
			while ( _feechangestack.Count > 0 )
				{
				FeeChangeRec rec = _feechangestack.Dequeue();
				DataRow row = ds.Tables[ 0 ].NewRow();
				row[ "symbol" ] = rec.Symbol;
				row[ "cpy" ] = rec.Counterparty;
				row[ "ccy" ] = rec.Ccy;
				row[ "fee" ] = rec.Fee;
				row[ "value_date" ] = rec.Valuedate;
				row[ "last_update" ] = rec.LastUpdateDateTime;

				if ( rec.Valuedate < _minvaldate )
					{
					_minvaldate = rec.Valuedate;
					if ( rec.LastUpdateDateTime < _minmoddate )
						{
						_minmoddate = rec.LastUpdateDateTime;
						}
					}

				ds.Tables[ 0 ].Rows.Add( row );
				}

			return ds.Tables[ 0 ];
			}

		private bool ComparePreviousFee( FeeChangeRec prev, FeeChangeRec cur )
			{
			bool res = false;
			if ( prev.Key.Equals( cur.Key ) ) return true;
			if ( (  prev.Symbol == cur.Symbol ) &&  
				 ( prev.Counterparty == cur.Counterparty ) && 
				  ( prev.Fee == cur.Fee ) ) 	res = true;
			return res;
			}

		public DataSet SwapNotionalPct( DateTime knowledgedate )
			{
			DataSet ds = new DataSet();
			string procname = Properties.Settings.Default.SwapNotionalPct;
			Int32 cmdTimeout = Properties.Settings.Default.CommandTimeout;
			try
				{
				PFSqlParameters myparams = new PFSqlParameters();
				myparams.AddInputParameter( "@knowledgedate", knowledgedate );
				PFSqlExecute excmd = new PFSqlExecute();
				excmd.FillDataSet( procname, true, _connstr, ds, cmdTimeout, myparams.Parameters );
				}
			catch ( SystemException ex )
				{
				_errormsg = ex.Message;
				throw ( new ApplicationException( "FormSwapQuery:SwapNotionalPct() Error Reading from SQL Server Database", ex ) );
				}
			return ds;
			}
		#endregion
		}
	}